#  Copyright (c) 1997-2019
#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
#  http://www.polymake.org
#
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the
#  Free Software Foundation; either version 2, or (at your option) any
#  later version: http://www.gnu.org/licenses/gpl.txt.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#-------------------------------------------------------------------------------

use strict;
use warnings;
use Benchmark qw(:all);
use POSIX qw(strftime);
use FileHandle;
use Benchmark qw(:all);

use application 'fan';

# cluster counts from 1 but files start with 0
my $job = @ARGV > 0 ? scalar($ARGV[0])-1 : undef;

script('motifs.pl'); # from this extension

my $codename="fill_polydb";

# absolute path @ TU Berlin
my $basedir = "/store/borel/lorenz/cluster/tropicalcubics/data/";

my $log_file="/tmp/fill_polydb.log";
my $host = $ENV{HOSTNAME} || "computer";

my $graph_iso_software=graph_iso_software();

my $db_db="Tropical";
my $db_collection="TropicalCubicsRaw";
my $db_type_information_key="default.TropicalCubicsRaw";

my $linecount = 10000;
my $verbose = 1;

sub fill_polydb($$;$) {
   my ($job_id,$count,$verbose) = @_;
   my $min_id = $job_id * $linecount;
   my $max_id = $min_id + $count;

   my $jobidstr = sprintf("%05d",$job_id);

   my $tdata="$basedir/3d3_reg_unimodular-id-$jobidstr.xz";
   my $cdata="$basedir/coeff/3d3_reg_unimodular-coeff-id-$jobidstr.xz";

   # mark start
   my $now = strftime "%a %b %e %H:%M:%S %Y", localtime();
   my $LOG = defined($job) ?
             FileHandle->new_from_fd(*STDOUT,">>") :
             FileHandle->new(">> $log_file");
   die "cannot write $log_file" unless defined($LOG);
   $LOG->autoflush(1);
   print $LOG "$$: [$now] started $codename @ $host\n";
   print $LOG "$$: min_id=$min_id max_id=$max_id\n";
   print $LOG "$$: graph iso software detected: ", $graph_iso_software, "\n";
   print $LOG "$$: reading $tdata\n";
   open my $TDATA, "xzcat $tdata|" or die "cannot read $tdata";
   print $LOG "$$: reading $cdata\n";
   open my $CDATA, "xzcat $cdata|" or die "cannot read $cdata";

   # start the clock
   my $t0=Benchmark->new();

   my $id=$min_id;
   my $i=0;
   my $db=PolyDB::Client::get_client();
   foreach my $tline (<$TDATA>) {
      my $cline = <$CDATA>;
      ++$id;
      chomp $tline;
      chomp $cline;
      my ($cid, $colon, @coeffs) = split " ", $cline;
      die "$tline [$id] not aligned to $cline [$cid]" unless $cid==$id;
      my $T = new IncidenceMatrix( new Set<Set>( eval($tline) ) ); # sort facets lexicographcally
      my $MW = new Vector<Int>(@coeffs);
      my $dsd = all_motifs($T,0,$MW);
      print $LOG " hash $id: ",$dsd->CANONICAL_HASH,"\n" if $i == 0;
      $dsd->name = $id;
      print STDERR " $id:", $dsd->N_ALL_MOTIFS, "\n" if $verbose && $id % 100 == 0;
      db_insert($dsd, db=>$db_db, collection=>$db_collection, type_information_key=>$db_type_information_key, client=>$db);
      ++$i;
      last if $id >= $max_id;
   }

   # stop the clock
   my $t1=Benchmark->new();
   my $td1=timediff($t1,$t0);
   $now = strftime "%a %b %e %H:%M:%S %Y", localtime();

   print $LOG "$$: [$now] processed $i lines\n", "$$: elapsed time for computation: ", timestr($td1), "\n";
   close $LOG or die "cannot close $log_file";
}


fill_polydb($job,$linecount,$verbose) if defined($job);

# Local Variables:
# mode: perl
# cperl-indent-level:3
# indent-tabs-mode:nil
# End:
